@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -18,34 +18,22 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
+#ifndef ROCRAND_H_
+#define ROCRAND_H_
+
 /** \rocrand_internal \addtogroup rocrandhost
  *
  *  @{
  */
 
-#ifndef ROCRAND_H_
-#define ROCRAND_H_
-
-#include <hip/hip_runtime.h>
-#include <hip/hip_fp16.h>
-
 #include "rocrand/rocrand_discrete_types.h"
+#include "rocrand/rocrand_hip_cpu.h"
 
-/// \cond ROCRAND_DOCS_MACRO
-#ifndef ROCRANDAPI
-#ifdef WIN32
-  #ifdef rocrand_EXPORTS
-  /* We are building this library */
-    #define ROCRANDAPI __declspec(dllexport)
-  #else
-    /* We are using this library */
-    #define ROCRANDAPI __declspec(dllimport)
-  #endif
-#else
-  #define ROCRANDAPI
-#endif
-#endif
-/// \endcond
+#include <hip/hip_fp16.h>
+#include <hip/hip_runtime.h>
+#include <hip/hip_vector_types.h>
+
+#include "rocrand/rocrandapi.h"
 
 #include "rocrand/rocrand_version.h"
 
@@ -86,17 +74,44 @@ typedef enum rocrand_status {
 /**
  * \brief rocRAND generator type
  */
-typedef enum rocrand_rng_type {
-    ROCRAND_RNG_PSEUDO_DEFAULT = 400, ///< Default pseudorandom generator
-    ROCRAND_RNG_PSEUDO_XORWOW = 401, ///< XORWOW pseudorandom generator
-    ROCRAND_RNG_PSEUDO_MRG32K3A = 402, ///< MRG32k3a pseudorandom generator
-    ROCRAND_RNG_PSEUDO_MTGP32 = 403, ///< Mersenne Twister MTGP32 pseudorandom generator
+typedef enum rocrand_rng_type
+{
+    ROCRAND_RNG_PSEUDO_DEFAULT       = 400, ///< Default pseudorandom generator
+    ROCRAND_RNG_PSEUDO_XORWOW        = 401, ///< XORWOW pseudorandom generator
+    ROCRAND_RNG_PSEUDO_MRG32K3A      = 402, ///< MRG32k3a pseudorandom generator
+    ROCRAND_RNG_PSEUDO_MTGP32        = 403, ///< Mersenne Twister MTGP32 pseudorandom generator
     ROCRAND_RNG_PSEUDO_PHILOX4_32_10 = 404, ///< PHILOX-4x32-10 pseudorandom generator
-    ROCRAND_RNG_QUASI_DEFAULT = 500,  ///< Default quasirandom generator
-    ROCRAND_RNG_QUASI_SOBOL32 = 501, ///< Sobol32 quasirandom generator
-    ROCRAND_RNG_QUASI_SOBOL64 = 504 ///< Sobol64 quasirandom generator
+    ROCRAND_RNG_PSEUDO_MRG31K3P      = 405, ///< MRG31k3p pseudorandom generator
+    ROCRAND_RNG_PSEUDO_LFSR113       = 406, ///< LFSR113 pseudorandom generator
+    ROCRAND_RNG_PSEUDO_MT19937       = 407, ///< Mersenne Twister MT19937 pseudorandom generator
+    ROCRAND_RNG_PSEUDO_THREEFRY2_32_20
+    = 408, ///< ThreeFry 32 bit state size 2 pseudorandom generator
+    ROCRAND_RNG_PSEUDO_THREEFRY2_64_20
+    = 409, ///< ThreeFry 64 bit state size 2 pseudorandom generator
+    ROCRAND_RNG_PSEUDO_THREEFRY4_32_20
+    = 410, ///< ThreeFry 32 bit state size 4 pseudorandom generator
+    ROCRAND_RNG_PSEUDO_THREEFRY4_64_20
+    = 411, ///< ThreeFry 64 bit state size 4 pseudorandom generator
+    ROCRAND_RNG_QUASI_DEFAULT           = 500, ///< Default quasirandom generator
+    ROCRAND_RNG_QUASI_SOBOL32           = 501, ///< Sobol32 quasirandom generator
+    ROCRAND_RNG_QUASI_SCRAMBLED_SOBOL32 = 502, ///< Scrambled Sobol32 quasirandom generator
+    ROCRAND_RNG_QUASI_SOBOL64           = 504, ///< Sobol64 quasirandom generator
+    ROCRAND_RNG_QUASI_SCRAMBLED_SOBOL64 = 505 ///< Scrambled Sobol64 quasirandom generator
+
 } rocrand_rng_type;
 
+/**
+ * \brief rocRAND generator ordering
+ */
+typedef enum rocrand_ordering
+{
+    ROCRAND_ORDERING_PSEUDO_BEST    = 100, ///< Best ordering for pseudorandom results
+    ROCRAND_ORDERING_PSEUDO_DEFAULT = 101, ///< Default ordering for pseudorandom results
+    ROCRAND_ORDERING_PSEUDO_SEEDED  = 102, ///< Fast lower quality pseudorandom results
+    ROCRAND_ORDERING_PSEUDO_LEGACY  = 103, ///< Legacy ordering for pseudorandom results
+    ROCRAND_ORDERING_PSEUDO_DYNAMIC = 104, ///< Adjust to the device executing the generator
+    ROCRAND_ORDERING_QUASI_DEFAULT  = 201 ///< n-dimensional ordering for quasirandom results
+} rocrand_ordering;
 
 // Host API function
 
@@ -108,10 +123,19 @@ typedef enum rocrand_rng_type {
  *
  * Values for \p rng_type are:
  * - ROCRAND_RNG_PSEUDO_XORWOW
+ * - ROCRAND_RNG_PSEUDO_MRG31K3P
  * - ROCRAND_RNG_PSEUDO_MRG32K3A
  * - ROCRAND_RNG_PSEUDO_MTGP32
  * - ROCRAND_RNG_PSEUDO_PHILOX4_32_10
+ * - ROCRAND_RNG_PSEUDO_LFSR113
+ * - ROCRAND_RNG_PSEUDO_THREEFRY2_32_20
+ * - ROCRAND_RNG_PSEUDO_THREEFRY2_64_20
+ * - ROCRAND_RNG_PSEUDO_THREEFRY4_32_20
+ * - ROCRAND_RNG_PSEUDO_THREEFRY4_64_20
  * - ROCRAND_RNG_QUASI_SOBOL32
+ * - ROCRAND_RNG_QUASI_SCRAMBLED_SOBOL32
+ * - ROCRAND_RNG_QUASI_SOBOL64
+ * - ROCRAND_RNG_QUASI_SCRAMBLED_SOBOL64
  *
  * \param generator - Pointer to generator
  * \param rng_type - Type of generator to create
@@ -165,6 +189,31 @@ rocrand_status ROCRANDAPI
 rocrand_generate(rocrand_generator generator,
                  unsigned int * output_data, size_t n);
 
+/**
+ * \brief Generates uniformly distributed 64-bit unsigned integers.
+ *
+ * Generates \p n uniformly distributed 64-bit unsigned integers and
+ * saves them to \p output_data.
+ *
+ * Generated numbers are between \p 0 and \p 2^64, including \p 0 and
+ * excluding \p 2^64.
+ *
+ * \param generator - Generator to use
+ * \param output_data - Pointer to memory to store generated numbers
+ * \param n - Number of 64-bit unsigned integers to generate
+ *
+ * \return
+ * - ROCRAND_STATUS_NOT_CREATED if the generator wasn't created \n
+ * - ROCRAND_STATUS_LAUNCH_FAILURE if a HIP kernel launch failed \n
+ * - ROCRAND_STATUS_LENGTH_NOT_MULTIPLE if \p n is not a multiple of the dimension
+ * of used quasi-random generator \n
+ * - ROCRAND_TYPE_ERROR if the generator can't natively generate 64-bit random numbers \n
+ * - ROCRAND_STATUS_SUCCESS if random numbers were successfully generated \n
+ */
+rocrand_status ROCRANDAPI rocrand_generate_long_long(rocrand_generator       generator,
+                                                     unsigned long long int* output_data,
+                                                     size_t                  n);
+
 /**
 * \brief Generates uniformly distributed 8-bit unsigned integers.
 *
@@ -461,7 +510,7 @@ rocrand_generate_poisson(rocrand_generator generator,
  *
  * If rocrand_initialize() was not called for a generator, it will be
  * automatically called by functions which generates random numbers like
- * rocrand_generate(), rocrang_generate_uniform() etc.
+ * rocrand_generate(), rocrand_generate_uniform() etc.
  *
  * \param generator - Generator to initialize
  *
@@ -497,9 +546,14 @@ rocrand_set_stream(rocrand_generator generator, hipStream_t stream);
  * - This operation resets the generator's internal state.
  * - This operation does not change the generator's offset.
  *
- * For a MRG32K3a generator seed value can't be zero. If \p seed is
- * equal zero and generator's type is ROCRAND_RNG_PSEUDO_MRG32K3A,
- * value \p 12345 is used as a seed instead.
+ * For an MRG32K3a or MRG31K3p generator the seed value can't be zero. If \p seed is
+ * equal to zero and generator's type is ROCRAND_RNG_PSEUDO_MRG32K3A or ROCRAND_RNG_PSEUDO_MRG31K3P,
+ * value \p 12345 is used as seed instead.
+ *
+ * For a LFSR113 generator seed values must be larger than 1, 7, 15,
+ * 127. The \p seed upper and lower 32 bits used as first and
+ * second seed value. If those values smaller than 2 and/or 8, those
+ * are increased with 1 and/or 7.
  *
  * \param generator - Pseudo-random number generator
  * \param seed - New seed value
@@ -512,6 +566,30 @@ rocrand_set_stream(rocrand_generator generator, hipStream_t stream);
 rocrand_status ROCRANDAPI
 rocrand_set_seed(rocrand_generator generator, unsigned long long seed);
 
+/**
+ * \brief Sets the seeds of a pseudo-random number generator.
+ *
+ * Sets the seed of the pseudo-random number generator. Currently only for LFSR113
+ *
+ * - This operation resets the generator's internal state.
+ * - This operation does not change the generator's offset.
+ *
+ * Only usable for LFSR113.
+ *
+ * For a LFSR113 generator seed values must be bigger than 1, 7, 15,
+ * 127. If those values smaller, than the requested minimum values [2, 8, 16, 128], then
+ * it will be increased with the minimum values minus 1 [1, 7, 15, 127].
+ *
+ * \param generator - Pseudo-random number generator
+ * \param seed - New seed value
+ *
+ * \return
+ * - ROCRAND_STATUS_NOT_CREATED if the generator wasn't created \n
+ * - ROCRAND_STATUS_TYPE_ERROR if the generator is a quasi-random number generator \n
+ * - ROCRAND_STATUS_SUCCESS if seed was set successfully \n
+ */
+rocrand_status ROCRANDAPI rocrand_set_seed_uint4(rocrand_generator generator, uint4 seed);
+
 /**
  * \brief Sets the offset of a random number generator.
  *
@@ -520,7 +598,8 @@ rocrand_set_seed(rocrand_generator generator, unsigned long long seed);
  * - This operation resets the generator's internal state.
  * - This operation does not change the generator's seed.
  *
- * Absolute offset cannot be set if generator's type is ROCRAND_RNG_PSEUDO_MTGP32.
+ * Absolute offset cannot be set if generator's type is ROCRAND_RNG_PSEUDO_MTGP32 or
+ * ROCRAND_RNG_PSEUDO_LFSR113.
  *
  * \param generator - Random number generator
  * \param offset - New absolute offset
@@ -528,11 +607,39 @@ rocrand_set_seed(rocrand_generator generator, unsigned long long seed);
  * \return
  * - ROCRAND_STATUS_NOT_CREATED if the generator wasn't created \n
  * - ROCRAND_STATUS_SUCCESS if offset was successfully set \n
- * - ROCRAND_STATUS_TYPE_ERROR if generator's type is ROCRAND_RNG_PSEUDO_MTGP32
+ * - ROCRAND_STATUS_TYPE_ERROR if generator's type is ROCRAND_RNG_PSEUDO_MTGP32 or
+ * ROCRAND_RNG_PSEUDO_LFSR113
  */
 rocrand_status ROCRANDAPI
 rocrand_set_offset(rocrand_generator generator, unsigned long long offset);
 
+/**
+ * \brief Sets the ordering of a random number generator.
+ *
+ * Sets the ordering of the results of a random number generator.
+ *
+ * - This operation resets the generator's internal state.
+ * - This operation does not change the generator's seed.
+ *
+ * \param generator - Random number generator
+ * \param order - New ordering of results
+ *
+ * The ordering choices for pseudorandom sequences are
+ * ROCRAND_ORDERING_PSEUDO_DEFAULT and
+ * ROCRAND_ORDERING_PSEUDO_LEGACY.
+ * The default ordering is ROCRAND_ORDERING_PSEUDO_DEFAULT, which is equal to
+ * ROCRAND_ORDERING_PSEUDO_LEGACY for now.
+ *
+ * For quasirandom sequences there is only one ordering, ROCRAND_ORDERING_QUASI_DEFAULT.
+ *
+ * \return
+ * - ROCRAND_STATUS_NOT_CREATED if the generator wasn't created \n
+ * - ROCRAND_STATUS_OUT_OF_RANGE if the ordering is not valid \n
+ * - ROCRAND_STATUS_SUCCESS if the ordering was successfully set \n
+ * - ROCRAND_STATUS_TYPE_ERROR if generator's type is not valid
+ */
+rocrand_status ROCRANDAPI rocrand_set_ordering(rocrand_generator generator, rocrand_ordering order);
+
 /**
  * \brief Set the number of dimensions of a quasi-random number generator.
  *
@@ -631,6 +738,6 @@ rocrand_destroy_discrete_distribution(rocrand_discrete_distribution discrete_dis
 }
 #endif /* __cplusplus */
 
-#endif // ROCRAND_H_
-
 /** @} */ // end of group rocrandhost
+
+#endif // ROCRAND_H_
