@@ -178,9 +178,9 @@ public:
     FQUALIFIERS
     void copy(const mtgp32_engine * m_engine)
     {
-        #if defined(__HIP_DEVICE_COMPILE__)
-        const unsigned int thread_id = hipThreadIdx_x;
-        for (int i = thread_id; i < MTGP_STATE; i += hipBlockDim_x)
+#if defined(__HIP_DEVICE_COMPILE__) || defined(USE_HIP_CPU)
+        const unsigned int thread_id = threadIdx.x;
+        for(int i = thread_id; i < MTGP_STATE; i += blockDim.x)
             m_state.status[i] = m_engine->m_state.status[i];
 
         if (thread_id == 0)
@@ -199,7 +199,7 @@ public:
             single_temper_tbl[thread_id] = m_engine->single_temper_tbl[thread_id];
         }
         __syncthreads();
-        #else
+#else
         this->m_state = m_engine->m_state;
         pos_tbl = m_engine->pos_tbl;
         sh1_tbl = m_engine->sh1_tbl;
@@ -210,7 +210,7 @@ public:
             temper_tbl[j] = m_engine->temper_tbl[j];
             single_temper_tbl[j] = m_engine->single_temper_tbl[j];
         }
-        #endif
+#endif
     }
 
     FQUALIFIERS
@@ -236,9 +236,9 @@ public:
     FQUALIFIERS
     unsigned int next()
     {
-        #if defined(__HIP_DEVICE_COMPILE__)
-        unsigned int t = hipThreadIdx_x;
-        unsigned int d = hipBlockDim_x;
+#if defined(__HIP_DEVICE_COMPILE__) || defined(USE_HIP_CPU)
+        unsigned int t   = threadIdx.x;
+        unsigned int d   = blockDim.x;
         int pos = pos_tbl;
         unsigned int r;
         unsigned int o;
@@ -254,17 +254,17 @@ public:
             m_state.offset = (m_state.offset + d) & MTGP_MASK;
         __syncthreads();
         return o;
-        #else
+#else
         return 0;
-        #endif
+#endif
     }
 
     FQUALIFIERS
     unsigned int next_single()
     {
-        #if defined(__HIP_DEVICE_COMPILE__)
-        unsigned int t = hipThreadIdx_x;
-        unsigned int d = hipBlockDim_x;
+#if defined(__HIP_DEVICE_COMPILE__) || defined(USE_HIP_CPU)
+        unsigned int t   = threadIdx.x;
+        unsigned int d   = blockDim.x;
         int pos = pos_tbl;
         unsigned int r;
         unsigned int o;
@@ -280,9 +280,9 @@ public:
             m_state.offset = (m_state.offset + d) & MTGP_MASK;
         __syncthreads();
         return o;
-        #else
+#else
         return 0;
-        #endif
+#endif
     }
 
 private:
@@ -514,9 +514,9 @@ unsigned int rocrand(rocrand_state_mtgp32 * state)
  * __global__
  * void generate_kernel(hiprandStateMtgp32_t * states, unsigned int * output, const size_t size)
  * {
- *      const unsigned int state_id = hipBlockIdx_x;
- *      unsigned int index = hipBlockIdx_x * hipBlockDim_x + hipThreadIdx_x;
- *      unsigned int stride = hipGridDim_x * hipBlockDim_x;
+ *      const unsigned int state_id = blockIdx.x;
+ *      unsigned int index = blockIdx.x * blockDim.x + threadIdx.x;
+ *      unsigned int stride = gridDim.x * blockDim.x;
  *
  *      __shared__ GeneratorState state;
  *      rocrand_mtgp32_block_copy(&states[state_id], &state);
